#!/usr/bin/env python
# -*- coding: utf-8 -*-

# rafi.mx build script
# ===

import subprocess
import re

rev_list = subprocess.check_output(['git', 'rev-list', '--all'])
commit_count = rev_list.decode('utf-8').count('\n')

with open('setup.py') as f:
    setup = f.read()

setup = re.sub(
    "MICRO_VERSION = '[0-9]+'",
    "MICRO_VERSION = '{}'".format(commit_count), setup)

major = re.search('MAJOR_VERSION = \'([0-9]+)\'', setup).groups()[0]
minor = re.search('MINOR_VERSION = \'([0-9]+)\'', setup).groups()[0]
micro = re.search('MICRO_VERSION = \'([0-9]+)\'', setup).groups()[0]
version = '{}.{}.{}'.format(major, minor, micro)

with open('setup.py', 'w') as f:
    f.write(setup)

with open('src/mx/__init__.py') as f:
    init = f.read()

with open('src/mx/__init__.py', 'w') as f:
    f.write(
        re.sub('__version__ = \'[0-9.]+\'',
               '__version__ = \'{}\''.format(version), init))



# -*- coding: utf-8 -*-
import re
import os
import subprocess
from .logger import Logger
from .tmux import Tmux

log = Logger()


class Git(object):
    _config = {}
    _repos = []
    _root = ''

    def __init__(self, config):
        """
        :param config: Dictionary with config schema
        """
        self._config = config
        self._root = self._config.get('dir') or os.getcwd()
        self._root = os.path.expanduser(self._root)

        # Collect normalized list of repositories in workspace
        for repo_name in self._config.get('repos', []):
            if isinstance(repo_name, str):
                repo = {
                    'name': repo_name,
                    'url': self._parse_repo_url(repo_name),
                    'dir': repo_name.split('/')[1]
                }
            else:
                repo = repo_name
                if 'url' not in repo:
                    repo['url'] = self._parse_repo_url(repo['name'])

            self._repos.append(repo)

    def _parse_repo_url(self, repo_name):
        """
        Complete full URL for short named repositories
        """
        url = None
        if 'github.com' not in repo_name:
            url = 'https://github.com/{}.git'.format(repo_name)
        return url or repo_name

    def clone(self):
        """
        Clone all repositories in project directory
        """
        log.echo(' [blue]::[reset] Fetching git index for project at [white]{}'
                 .format(self._root))
        if not os.path.isdir(self._root):
            os.makedirs(self._root)
        os.chdir(self._root)

        for repo in self._repos:
            log.echo(' [blue]::[reset] Cloning [white]{} [boldblack]@ {}'
                     .format(repo['name'], repo['url']))
            subprocess.check_output(
                ['git', 'clone', repo['url']],
                stderr=subprocess.STDOUT)

    def fetch(self):
        """
        Iterate through all repositories and run git fetch with rich output
        """
        log.echo(' [blue]::[reset] Fetching git index for project at [white]{}'
                 .format(self._root))
# Debugging:
#        with open('output_example.txt') as f:
#            output = f.read()
#        self._parse_git_fetch(output)
#        return
        for repo in self._repos:
            log.echo(' [blue]::[reset] Fetching [white]{} [boldblack]@ {}'
                     .format(repo['name'], repo['url']))
            os.chdir(self._root)
            os.chdir(repo['dir'])
            output = subprocess.check_output(
                ['git', 'fetch', '--all', '--tags', '--prune'],
                stderr=subprocess.STDOUT)
            self._parse_git_fetch(output.decode('utf_8'))

    def _parse_git_fetch(self, output):
        """
        Parse and beautify git's raw fetch summary

        :param output: Git's raw fetch output
        """
        branches = {'created': [], 'updated': []}
        tags = {'created': [], 'updated': []}
        deleted = []
        # Example matches:
        #
        #  * [new branch]      1.34.3     -> gogs/1.34.3
        #  bc23688..8be82ed  develop    -> gogs/develop
        #  * [new tag]         0.9.1      -> 0.9.1
        #  - [tag update]      1.19.0     -> 1.19.0
        #  - [tag update]      1.19.1     -> 1.19.1
        #  - [tag update]      1.19.2     -> 1.19.2
        #  * [new tag]         1.19.3     -> 1.19.3
        #  * [new tag]         1.19.4     -> 1.19.4
        #  x [deleted]         (none)     -> origin/foobar
        regex = re.compile(
            r'^\s+([-+*x\ ])\s+\[?([\w\ \.]+)\]?'
            '\s{2,}([^\s]+)\s{2,}->\s(.*)$',
            flags=re.MULTILINE
        )
        for match in regex.finditer(output):
            remote = match.group(4)
            if match.group(2).find('new') > -1:
                action = 'created'
            else:
                action = 'updated'

            if match.group(2) == 'deleted':
                deleted.append(remote)
            if match.group(2).find('tag') > -1:
                tags[action].append(remote)
            else:
                branches[action].append(remote)

        for action in ['created', 'updated']:
            if tags[action] or branches[action]:
                log.echo('   [{}]::[reset] {}'
                         ' {}[yellow]([boldyellow]{}[yellow])[reset]'
                         ' {}[yellow]([boldred]{}[yellow])[reset]'
                         .format(
                             'green' if action == 'created' else 'yellow',
                             action.title(),
                             'tags: ' if tags[action] else '',
                             ', '.join(tags[action]),
                             'branches: ' if branches[action] else '',
                             ', '.join(branches[action])))
        if deleted:
            log.echo('   [red]::[reset] Deleted:'
                     ' [yellow]([boldred]{}[yellow])[reset]'
                     .format(', '.join(deleted)))

    def status(self):
        """
        Iterate through repositories and display a colorful status
        """
        session_name = self._config.get('name')
        is_on = Tmux().has_session(session_name)
        log.echo(' [blue]::[reset] Session [boldyellow]{}[reset]: {}'
                 .format(session_name,
                         '[boldgreen]on' if is_on else '[boldred]off'))

        # Compatibility for python 2.x, doesn't have subprocess.DEVNULL
        DEVNULL = open(os.devnull, 'wb')

        for repo in self._repos:
            os.chdir(self._root)
            os.chdir(repo['dir'])

            try:
                output = subprocess.check_output(
                    ['git', 'symbolic-ref', '-q', 'HEAD'],
                    stderr=DEVNULL)
                detached = False
            except subprocess.CalledProcessError:
                detached = True

            output = subprocess.check_output(['git', 'diff', '--shortstat'])
            modified = re.match(r'^\s*(\d)', output.decode('utf_8'))
            modified = '≠' + str(modified.group(1)) if modified else ''

            output = subprocess.check_output(
                ['git', 'ls-files', '--others', '--exclude-standard'],
                stderr=DEVNULL).decode('utf-8')
            untracked = len(output.split('\n')) - 1
            untracked = '?' + str(untracked) if untracked > 0 else ''

            current = subprocess.check_output([
                'git', 'log', '-1', '--color=always',
                '--format=%C(auto)%D %C(black bold)(%aN %ar)%Creset'
            ]).decode('utf-8').strip()

            if detached:
                position = 'detach'
            else:
                output = subprocess.check_output([
                    'git', 'rev-parse', '--abbrev-ref', 'HEAD'])
                branch = output.decode('utf-8').strip()
                upstream = 'origin/{}'.format(branch)
                try:
                    output = subprocess.check_output(
                        ['git', 'rev-parse', '--abbrev-ref', '@{upstream}'],
                        stderr=DEVNULL)
                    upstream = output.decode('utf-8').strip()
                except subprocess.CalledProcessError:
                    pass

                position = ''
                try:
                    output = subprocess.check_output([
                        'git', 'rev-list', '--left-right',
                        branch, '...', upstream
                    ], stderr=subprocess.STDOUT).decode('utf-8')
                    ahead = len(re.findall(r'\<', output))
                    behind = len(re.findall(r'\>', output))
                    position = '{}{}'.format(
                        '▲' + str(ahead) if ahead else '',
                        '▼' + str(behind) if behind else '',
                    )
                except subprocess.CalledProcessError:
                    position = 'n/a'

            name = repo['name']
            if repo['name'].split('/')[1] != repo['dir']:
                name = repo['dir']

            log.echo('   [white]{:>30} '
                     ' [boldred]{:3} [boldblue]{:3} [boldmagenta]{:7}'
                     ' [reset]{}'
                     .format(
                         name, modified, untracked,
                         position, current))

        DEVNULL.close()

    @staticmethod
    def is_git_repo():
        """
        Check if current directory is a Git repository
        """
        try:
            subprocess.check_output(
                ['git', 'rev-parse', '--is-inside-work-tree'],
                stderr=subprocess.STDOUT)
            return True
        except subprocess.CalledProcessError:
            return False

    @staticmethod
    def get_remote_url():
        """
        Returns the current directory's Git repository remote URL
        """
        try:
            url = subprocess.check_output(
                ['git', 'config', '--get', 'remote.origin.url'],
                stderr=subprocess.STDOUT)
            return url.decode('utf-8').strip()
        except subprocess.CalledProcessError:
            return False



# -*- coding: utf-8 -*-
import json
import os
import subprocess
from .logger import Logger

log = Logger()


class TmuxException(Exception):
    pass


class Tmux(object):
    """
    Tmux controller
    """
    def command(self, cmd, formats=None, many=False):
        """
        Send custom Tmux command and return rich information

        :param cmd:
        :param formats:
        :param many:
        :return:
        """
        cmd.insert(0, 'tmux')
        if formats:
            fmt = '{'
            for key in formats:
                fmt += ''.join(['"', key, '": ', '"#{', key, '}", '])
            fmt = fmt[0:-2] + '}'
            cmd.append('-F')
            cmd.append(fmt)

        try:
            process = subprocess.Popen(
                cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            process.wait()
            stdout = process.stdout.read()
            process.stdout.close()
            stderr = process.stderr.read()
            process.stderr.close()
            if stdout:
                lines = ','.join(stdout.decode('utf_8').split('\n')) \
                    .rstrip(',')
                stdout = json.loads('[' + lines + ']' if many else lines)
            if stderr:
                stderr = stderr.decode('utf_8').strip()

            return stdout, stderr
        except ValueError:
            raise TmuxException('Unable to serialize Tmux\'s response, '
                                'please report bug.')
        except Exception:
            raise TmuxException('Unable to execute Tmux, aborting.')

    def within_session(self):
        """
        Returns true if current within a Tmux session
        """
        return os.environ.get('TMUX') != ''

    def has_session(self, session_name):
        """
        Returns true if specified session currently exists

        :param session_name: The session name to match
        """
        try:
            cmds = ['tmux', 'has-session', '-t', session_name]
            # Compatibility code, python 2.x doesn't have subprocess.DEVNULL
            with open(os.devnull, 'wb') as DEVNULL:
                code = subprocess.check_call(cmds, stderr=DEVNULL)
        except subprocess.CalledProcessError as e:
            code = e.returncode
        return code == 0

    def new_session(self, session_name, win_name=''):
        """
        Create a new Tmux session

        :param session_name: New session's name
        :param win_name: The window's name within the new session
        :return: (session, window, pane)
        """
        cmds = ['new-session', '-Pd', '-s', session_name]
        if win_name:
            cmds.extend(['-n', win_name])

        output, errors = self.command(
            cmds,
            ['session_id', 'session_name', 'session_windows',
             'window_id', 'window_index', 'pane_index', 'pane_id'])

        if errors:
            raise TmuxException(errors)
        session = {}
        window = {}
        pane = {}
        for k, v in output.items():
            short_name = k.split('_')[1]
            if k.startswith('window_'):
                window[short_name] = v
            elif k.startswith('pane_'):
                pane[short_name] = v
            else:
                session[short_name] = v

        return session, window, pane

    def new_window(self, session_name, win_name=''):
        """
        Create a new Tmux window

        :param session_name: Target session name
        :param win_name: The new window's name
        :return: (window, pane)
        """
        cmds = ['new-window', '-Pd', '-t', session_name]
        if win_name:
            cmds.extend(['-n', win_name])

        output, errors = self.command(
            cmds,
            ['window_id', 'window_name', 'window_panes', 'window_active',
             'window_index', 'window_layout', 'pane_index', 'pane_id'])

        if errors:
            raise Exception('Error creating window: {}'.format(errors))
        window = {}
        pane = {}
        for k, v in output.items():
            short_name = k.split('_')[1]
            if k.startswith('pane_'):
                pane[short_name] = v
            else:
                window[short_name] = v

        return window, pane

    def new_pane(self, session_name, window_id, pane_id):
        """
        Create a new Tmux pane

        :param session_name: Target session name
        :param window_id: Window to split from
        :param pane_id: Pane to split from
        :return: Pane information
        """
        output, errors = self.command(
            ['split-window', '-h', '-P', '-t',
             '{}:{}.{}'.format(session_name, window_id, str(pane_id))],
            ['pane_id', 'pane_index', 'pane_active', 'pane_current_path',
             'pane_start_command', 'pane_current_command', 'pane_title'])
        if errors:
            raise TmuxException(errors)
        pane = {}
        for k, v in output.items():
            short_name = k.split('_')[1]
            pane[short_name] = v
        return pane

    def kill_session(self, session_name):
        """
        Kill a specified Tmux session
        """
        return self.command(['kill-session', '-t', session_name])

    def set_layout(self, session_name, win_name, layout=None):
        """
        Sets a Tmux session's specific window to a different layout

        :param session_name: Target session name
        :param win_name: Target window name
        :param layout: The layout name (even-horizontal, even-vertical,
                                        main-horizontal, main-vertical, tiled)
        """
        return self.command([
            'select-layout', '-t',
            '{}:{}'.format(session_name, win_name), layout or 'tiled'
        ])

    def send_keys(self, session_name, win_name, pane_index, cmd, enter=True):
        """
        Sends a Tmux session custom keys

        :param session_name: Target session name
        :param win_name: Target window name
        :param pane_index: Target pane index
        :param cmd: The string to enter
        :param enter: Finish with a carriage-return?
        """
        if cmd:
            return self.command([
                'send-keys', '-Rt',
                '{}:{}.{}'.format(session_name, win_name, str(pane_index)),
                cmd, 'C-m' if enter else ''
            ])

    def attach(self, session_name):
        """
        Attach to an existing Tmux session

        :param session_name: Target session name
        """
        if session_name:
            cmd = 'switch-client' if self.within_session() \
                else 'attach-session'
            return self.command([cmd, '-t', session_name])

    def get_windows(self, session_name):
        """
        Retrieve information for all windows in a session

        :param session_name: Target session name
        """
        return self.command(
            ['list-windows', '-t', session_name],
            ['window_id', 'window_name', 'window_panes', 'window_active',
             'window_index', 'window_layout'],
            many=True)

    def get_panes(self, session_name, window_name):
        """
        Retrieve information for all panes in a window

        :param session_name: Target session name
        :param window_name: Target window name
        """
        return self.command(
            ['list-panes', '-t', ':'.join([session_name, window_name])],
            ['pane_id', 'pane_active', 'pane_index', 'pane_start_command',
             'pane_current_command', 'pane_current_path', 'pane_title'],
            many=True)



# -*- coding: utf-8 -*-
"""Workspace/project-oriented tmux/git personal assistant.

Working on multiple git repositories and juggling tmux sessions can be tedious.
`mx` tries to help with that.
"""
__package__ = 'mx'
__license__ = 'MIT'
__version__ = '0.2.17'
__author__ = __maintainer__ = 'Rafael Bodill'
__email__ = 'justrafi@gmail'



# -*- coding: utf-8 -*-
import yaml
import argparse
import os
import sys
from . import __version__
from .logger import Logger
from .git import Git
from .tmux import TmuxException
from .workspace import Workspace, WorkspaceException

WORKSPACE_COMMANDS = ['attach', 'start', 'stop', 'ls', 'init']
GIT_COMMANDS = ['clone', 'fetch', 'status']


def main():
    """
    Start main program: Parse user arguments and take action
    """
    parser = argparse.ArgumentParser(
        description='mx: Orchestrate tmux sessions and git projects')

    parser.add_argument('action', type=str, nargs='?', default='start',
                        choices=WORKSPACE_COMMANDS + GIT_COMMANDS,
                        help='an action for %(prog)s (default: %(default)s)')
    parser.add_argument('session', type=str, nargs='?',
                        help='session for %(prog)s to load'
                        ' (default: current directory\'s .mx.yml)')
    parser.add_argument('-c', '--config', type=str, default='.mx.yml',
                        help='workspace yml config file'
                             ' (default: %(default)s)')
    parser.add_argument('-v', action='version',
                        version='%(prog)s {}'.format(__version__))

    args = parser.parse_args()

    # Compute cache pool path
    cache_dir = os.environ.get('XDG_CACHE_HOME',
                               os.path.join(os.environ.get('HOME'), '.cache'))
    pool_dir = os.path.join(cache_dir, 'mx')

    if args.session:
        # Load session from cache pool (symlinks)
        cfg_path = os.path.join(pool_dir, '{}.yml'.format(args.session))
    else:
        # Load session from cli option (or default value)
        cfg_path = os.path.realpath(args.config)

    log = Logger()
    if not os.path.isfile(cfg_path):
        if args.action == 'init':
            schema = Workspace.initialize(os.getcwd())
            with open(cfg_path, 'w') as cfg_file:
                cfg_file.write(
                    yaml.safe_dump(schema, default_flow_style=False))
            args.action = 'status'
        else:
            log.echo('[red]ERROR: [reset]Unable to find [white]{}'
                     .format(cfg_path))
            sys.exit(2)

    try:
        # Read configuration and run the requested action
        with open(cfg_path, 'r') as stream:
            config = yaml.load(stream)
        run(config, args.action)

        # Save session symlink in cache pool
        if not os.path.isdir(pool_dir):
            os.makedirs(pool_dir)
        link = os.path.join(pool_dir, '{}.yml'.format(config.get('name')))
        if not os.path.isfile(link):
            os.symlink(cfg_path, link)

    except (WorkspaceException, TmuxException) as e:
        if hasattr(e, '__context__') and e.__context__:
            log.echo(' -> {}'.format(e.__context__))
        if hasattr(e, 'errors'):
            log.echo('[red]{}: [reset]{}'.format(e.message, e.errors))
        else:
            log.echo('[red]Raw error: [reset]{}'.format(str(e)))
        sys.exit(3)


def run(config, action):
    """
    Execute tmux or git workspace related actions
    """
    if action in WORKSPACE_COMMANDS:
        workspace = Workspace(config)
        getattr(workspace, action)()

    # Or, git related actions
    elif action in GIT_COMMANDS:
        git = Git(config)
        getattr(git, action)()



# -*- coding: utf-8 -*-
import re
import sys
import os
from .logger import Logger
from .tmux import Tmux
from .git import Git

log = Logger()


class WorkspaceException(Exception):
    def __init__(self, message, errors=''):
        super(WorkspaceException, self).__init__(message)
        self.errors = errors
        self.message = message


class Workspace(object):
    _tmux = Tmux()
    _config = {}
    _name = ''
    _root = ''
    _venv = []
    _session = {}
    _windows = []

    def __init__(self, config=None):
        """
        :param tmux: Tmux class instance
        """
        self.set_config(config)

    def set_config(self, config):
        self._config = config
        self._name = self._config.get('name')
        self._root = self._config.get('dir') or os.getcwd()
        self._root = os.path.expanduser(self._root)

        # Prepare a virtualenv source command, if any
        # The specified virtualenv path can be absolute or relative
        # to the workspace's root directory.
        venv = self._config.get('venv')
        if venv:
            if self._root and not os.path.isabs(venv):
                venv = os.path.join(self._root, venv)
            self._venv = [
                ' source "{}"'.format(os.path.join(venv, 'bin/activate'))]

    def start(self):
        """
        Create Tmux windows from `windows` configuration in YML

        :return Collection of window information
        """
        if self._root:
            if not os.path.isdir(self._root):
                raise WorkspaceException('Directory does not exist',
                                         self._root)
            os.chdir(self._root)
        for window in self._config.get('windows', []):
            # Normalize window schema definition, a window definition:
            #   - string - window name
            #   - key/value - window name / command
            #   - dictionary - { panes: [], layout: '', post_cmd: '' / [] }
            if isinstance(window, str):
                name = window
            else:
                name = next(iter(window.keys()))
                window = window[name] or {}

            if isinstance(window, str):
                panes = [window]
                window = {}
            else:
                panes = window.get('panes', [])

            # Normalize post commands, can be string or list
            post_cmds = window.get('post_cmd', [])
            post_cmds = [post_cmds] if isinstance(post_cmds, str) \
                else post_cmds

            # Create session, window, and panes
            # with a layout and post commands.
            self._windows.append(
                self.create_window(
                    name, panes, post_cmds, window.get('layout')))

        self.attach()
        return self._windows

    def stop(self, name=None):
        """
        Kill a workspace session

        :param name: Session name
        """
        self._tmux.kill_session(name or self._name)

    def attach(self, name=None):
        """
        Attach to a workspace session

        :param name: Session name
        """
        self._tmux.attach(name or self._name)

    def ls(self):
        """
        List windows and panes for a session
        """
        windows, errors = self._tmux.get_windows(self._name)
        if errors:
            raise WorkspaceException('Unable to list windows', errors)
        log.echo(' [blue]::[reset] Windows:')
        log.echo(repr(windows))

        for window in windows:
            win_id = window['window_id']
            panes, errors = self._tmux.get_panes(self._name, win_id)
            if errors:
                raise WorkspaceException('Unable to list panes', errors)
            log.echo(' [blue]::[reset] Window "{}" panes:'.format(win_id))
            log.echo(repr(panes))

    def create_window(self, name, panes, post_cmds, layout=None):
        """
        Create Tmux window and panes from configuration

        :param name:
        :param panes:
        :param post_cmds:
        :param layout:
        :return
        """
        if len(self._windows) > 0:
            window, pane = self._tmux.new_window(self._name, name)
        else:
            self._session, window, pane = \
                self._tmux.new_session(self._name, name)
            if not self._session:
                print('Error creating session buddy.')
                sys.exit(1)

        session_name = self._session['name']
        window['panes'] = []
        for pane_schema in panes:
            if len(window['panes']) > 0:
                pane = self._tmux.new_pane(session_name, name, pane['index'])
            window['panes'].append(pane)

            # Run commands+post-commands, and activate virtualenv
            cmds = next(iter(pane_schema.values())) \
                if isinstance(pane_schema, dict) else [pane_schema]
            cmds = self._venv + cmds + post_cmds
            for cmd in cmds:
                self._tmux.send_keys(session_name, name, pane['index'], cmd)

        self._tmux.set_layout(session_name, name, layout)
        return window

    @staticmethod
    def initialize(root_dir):
        """
        Initialize a new workspace .mx.yml file
        """
        dirs = next(os.walk(root_dir))[1]
        repos = []
        skipped = []
        for directory in dirs:
            os.chdir(os.path.join(root_dir, directory))
            if Git.is_git_repo():
                log.echo(' [blue]::[reset] Found directory'
                         ' `[white]{}[reset]`'.format(directory))
                url = Git.get_remote_url()
                name = url.replace(':', '/')
                name = re.sub('.git$', '', name)
                name = '/'.join(name.split('/')[-2:])
                if name.split('/')[1] == directory:
                    repos.append(name)
                else:
                    repos.append({'dir': directory, 'name': name})
            else:
                skipped.append(directory)

        log.echo(' [blue]::[yellow] Skipped[reset]'
                 ' non-git repositories: {}'.format(', '.join(skipped)))

        return {
            'name': os.path.basename(root_dir),
            'dir': root_dir,
            'repos': repos
        }



# -*- coding: utf-8 -*-
import sys
import re


class Logger(object):
    """
    Terminal logger with color-support
    """
    _is_tty = sys.stdout.isatty()
    _colors = {'reset': 0, 'black': 30, 'white': 37,
               'cyan': 36, 'magenta': 35, 'blue': 34,
               'yellow': 33, 'green': 32, 'red': 31}

    def echo(self, *args):
        """
        Prints text to terminal with color codes
        Example:
          log.echo('[green]hey [boldred]there!')

        :param args: Multiple string messages
        """
        for arg in args:
            msg = re.sub(r'\[(bold)?([a-z]+)\]', self._colorize, arg)
            print(''.join([msg, '\x1b[0m']))

    def _colorize(self, match):
        if not self._is_tty:
            return ''
        attr = ['1' if match.group(1) == 'bold' else '0']
        attr.append(str(self._colors[match.group(2)]))
        return '\x1b[{}m'.format(';'.join(attr))



from setuptools import setup, find_packages
from src import mx

MAJOR_VERSION = '0'
MINOR_VERSION = '2'
MICRO_VERSION = '17'
VERSION = '{}.{}.{}'.format(MAJOR_VERSION, MINOR_VERSION, MICRO_VERSION)

setup(
    name='rafi.mx',
    version=VERSION,
    description='Workspace/project-oriented tmux/git personal assistant.',
    long_description=mx.__doc__,
    author=mx.__author__,
    author_email=mx.__email__,
    license=mx.__license__,
    url='http://github.com/rafi/mx',
    keywords='tmux git workspace project assistant',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=['PyYAML'],
    extras_requires=['pytest', 'mock'],
    platforms='any',
    zip_safe=False,
    entry_points={
        'console_scripts': ['mx = mx.cli:main']
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Customer Service',
        'Intended Audience :: System Administrators',
        'Operating System :: Microsoft',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Debuggers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities'
    ]
)
